import os
import numpy as np
from scipy.interpolate import UnivariateSpline
from scipy.interpolate import interp1d

wdir = '/'.join(os.path.realpath(__file__).split("/")[:-1])+'/Grids/'

def GetBRfunctions(spline_deg = 3):


    #### SM BR(h --> ff) ####

    BR_ferm_file = open(wdir+'BR_fermions.dat')
    BR_ferm_file.next()
    BR_ferm_file.next()
    ferm_grid = {"bb": [], "tautau": [], "cc": [], "mumu": [], "ss": []}
    hmass = []
    for line in BR_ferm_file:
        line = line.strip("\n").split()
        hmass.append(float(line[0]))
        ferm_grid["bb"].append(float(line[1]))
        ferm_grid["tautau"].append(float(line[4]))
        ferm_grid["cc"].append(float(line[10]))
        ferm_grid["mumu"].append(float(line[7]))
        ferm_grid["ss"].append(float(line[13]))

    BR_bb = UnivariateSpline(hmass, ferm_grid["bb"], k = spline_deg, s = 0)
    BR_tautau = UnivariateSpline(hmass, ferm_grid["tautau"], k = spline_deg, s = 0)
    BR_cc = UnivariateSpline(hmass, ferm_grid["cc"], k = spline_deg, s = 0)
    BR_mumu = UnivariateSpline(hmass, ferm_grid["mumu"], k = spline_deg, s = 0)
    BR_ss = UnivariateSpline(hmass, ferm_grid["ss"], k = spline_deg, s = 0)

    BR_ferm_file.close()


    #### SM BR(h --> VV) ####


    BR_gauge_file = open(wdir+'BR_gauge.dat')
    BR_gauge_file.next()
    BR_gauge_file.next()
    gauge_grid = {"gg": [], "GaGa": [], "ZGa": [], "WW": [], "ZZ": []}
    hmass = []
    for line in BR_gauge_file:
        line = line.strip("\n").split()
        hmass.append(float(line[0]))
        gauge_grid["gg"].append(float(line[1]))
        gauge_grid["GaGa"].append(float(line[4]))
        gauge_grid["ZGa"].append(float(line[7]))
        gauge_grid["WW"].append(float(line[10]))
        gauge_grid["ZZ"].append(float(line[13]))

    BR_gg = UnivariateSpline(hmass, gauge_grid["gg"], k = spline_deg, s = 0)
    BR_GaGa = UnivariateSpline(hmass, gauge_grid["GaGa"], k = spline_deg, s = 0)
    BR_ZGa = UnivariateSpline(hmass, gauge_grid["ZGa"], k = spline_deg, s = 0)
    BR_WW = UnivariateSpline(hmass, gauge_grid["WW"], k = spline_deg, s = 0)
    BR_ZZ = UnivariateSpline(hmass, gauge_grid["ZZ"], k = spline_deg, s = 0)
    BR_gauge_file.close()

    BR = {"BR_bb": BR_bb, "BR_cc": BR_cc, "BR_tautau": BR_tautau, "BR_mumu": BR_mumu, "BR_ss": BR_ss, "BR_WW": BR_WW, "BR_ZZ": BR_ZZ, "BR_GaGa": BR_GaGa, "BR_gg": BR_gg, "BR_ZGa": BR_ZGa}

    return BR



def GetEffVVHfunctions(spline_deg = 3):

    #### SM efficiencies h --> WW & h --> ZZ ####

    WHZH_xsec_file = open(wdir+'WH_ZH_VBF_xsec.dat')
    eff_grid = {"WH": [], "ZH": [], "VH": [], "VBF": []}
    hmass = []
    for line in WHZH_xsec_file:
        line = line.strip("\n").split()
        hmass.append(float(line[0]))
        eff_grid["WH"].append(float(line[1])/(float(line[1])+float(line[2])))
        eff_grid["ZH"].append(float(line[2])/(float(line[1])+float(line[2])))
        eff_grid["VH"].append((float(line[1])+float(line[2]))/(float(line[1])+float(line[2])+float(line[3])))
        eff_grid["VBF"].append(float(line[3])/(float(line[1])+float(line[2])+float(line[3])))
    

    eff_WH = UnivariateSpline(hmass, eff_grid["WH"], k = spline_deg, s = 0)
    eff_ZH = UnivariateSpline(hmass, eff_grid["ZH"], k = spline_deg, s = 0)
    eff_VH = UnivariateSpline(hmass, eff_grid["VH"], k = spline_deg, s = 0)
    eff_VBF = UnivariateSpline(hmass, eff_grid["VBF"], k = spline_deg, s = 0)
    WHZH_xsec_file.close()

    effWZ = {"eff_WH": eff_WH, "eff_ZH": eff_ZH, "eff_VH": eff_VH, "eff_VBF": eff_VBF}

    return effWZ
