#! /usr/bin/env python
#########################################################
# Lilith
#
# Routine example to create a grid of -2LogL
# Model: (C_gamma, C_gluon)
#
#########################################################

import lilith, tools.genXML
import numpy

# sys.path.append("/path/to/lilith") is needed if scan_CMS-14009_CGaCg.py
# is not in the same directory

# Output file name
mylresults = "results/scan_CGaCg.dat"
# Choice of experimental data
myexpinput = "data/CMS14009.list"
# Lilith precision
myprecision = "BEST-QCD"
# Higgs mass to be tested
myhmass = 125.03

verbose = False

# Range of parameters, number of steps
CGa_par =   [0., 2., 200]
Cg_par  =   [0., 2., 200]


############################################################

# Create a Lilith object
LHcalc = lilith.Lilith(verbose)

# Read experimental data
LHcalc.SetExpInput(myexpinput)

# Open output file
fresults = open(mylresults, 'w')

lmin=10000

# Make a grid of -2LogL
for CGa in numpy.linspace(CGa_par[0], CGa_par[1], CGa_par[2]):
    fresults.write('\n')
    for Cg in numpy.linspace(Cg_par[0], Cg_par[1], Cg_par[2]):
        myXML_user_input = tools.genXML.redC(myhmass, CGaGa=CGa, Cgg=Cg, precision=myprecision)
        l = LHcalc.GetLikelihood(myXML_user_input)[0]
        if l< lmin:
            lmin=l
            CGamin=CGa
            Cgmin=Cg
        fresults.write('%.5f    '%CGa +'%.5f    '%Cg + '%.5f     '%l + '\n')

fresults.close()

# Print best-fit point
print "C_gamma:", CGamin, "C_gluon:", Cgmin, "-2LogL min:", lmin
